@@ -1,18 +1,18 @@
 /* setboot - tell OS/2 Boot Manager what system to load at next boot */
 
 /* 
- * Copyright 1994 Chris Smith
+ * Copyright 1994, 1998 Chris Smith
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
  * provided that the above copyright notice appears in all copies and that
  * both that copyright notice and this permission notice appears in
- * supporting documentation.  I make no representations about the
+ * supporting documentation.  I make no representation about the
  * suitability of this software for any purpose.  It is provided "as is"
  * without express or implied warranty.
  */
 
-#ifndef __linux__
+#ifndef __unix__
 #define __dos__ 1
 #endif
 
@@ -21,13 +21,18 @@
 #include <stdarg.h>
 #include <string.h>
 
-#if __linux__
+#if __unix__
 #include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
 #endif
 
+#if __linux__
+#include <sys/ioctl.h>
+#include <linux/hdreg.h>
+#endif
+
 #if __dos__
 #include <bios.h>
 #include <dos.h>
@@ -35,9 +40,9 @@
 
 #include "bootmgr.h"
 
-#if __linux__
+#if __unix__
 
-/* On Linux, i/o is to /dev/hda and friends, a drive handle is an fd,
+/* On Unix, i/o is to /dev/hda and friends, a drive handle is an fd,
    and disk addresses are sector numbers, 0-origin. */
 
 typedef unsigned drive;
@@ -115,7 +120,7 @@ void put1 (drive dev, disk_addr addr, void *buf);
 void no_boot_manager (void);
 void fatal (char *fmt, ...);
 
-#if __linux__
+#if __unix__
 drive open_drive (char *dev);
 drive open_boot_drive (void);
 void fatal_perror (char *fmt, ...);
@@ -188,11 +193,30 @@ void read_partition_info ()
   int n;
   drive d;
 
-#if __linux__
+#if __unix__
   char dev[16];
 
   for (n = 0; ; n++) {
+#if __linux__
     sprintf (dev, "/dev/hd%c", n + 'a');
+    {
+      /* check if this /dev/hdx actually is a hard disk.
+	 (Linus must have been asleep the day he accepted THAT code.)
+	 [thanks to Georgatos Photis, gef@ceid.upatras.gr, for finding 
+	 this problem.] */
+      struct hd_geometry g;
+      int fd, rv;
+      fd = open (dev, 0);
+      if (fd != -1) {
+	rv = ioctl (fd, HDIO_GETGEO, &g);
+	close (fd);
+	if (rv == -1 && errno == EINVAL)
+	  continue;
+      }
+    }
+#else /* bsd */
+    sprintf (dev, "/dev/wd%dd", n);
+#endif
     if (d = open_drive (dev))
       read_partitions (d);
     else
@@ -200,7 +224,11 @@ void read_partition_info ()
   }
 
   for (n = 0; ; n++) {
+#if __linux__
     sprintf (dev, "/dev/sd%c", n + 'a');
+#else /* bsd */
+    sprintf (dev, "/dev/sd%dd", n);
+#endif
     if (d = open_drive (dev))
       read_partitions (d);
     else
@@ -264,7 +292,7 @@ void read_partitions (drive dev)
 
     for (log = ext, logical_partno = 4; log != 0; logical_partno++) {
       
-#if __linux__
+#if __unix__
       if (log == ext)
 	p = get1 (dev, ext->secno);
       else
@@ -330,7 +358,7 @@ void read_boot_manager ()
 
   /* Record the addresses of boot manager sectors 0, 1, 3. */
 
-#if __linux__
+#if __unix__
   bm_dev = open_boot_drive ();
   bm_0 = info[n].secno;
   bm_1 = bm_0 + 1;
@@ -529,7 +557,7 @@ void fatal (char *fmt, ...)
   exit (1);
 }
 
-#if __linux__
+#if __unix__
 
 drive open_drive (char *dev)
 {
@@ -538,7 +566,7 @@ drive open_drive (char *dev)
 
   fd = open (dev, 0);
   if (fd < 0) {
-    if (errno == ENODEV || errno == ENOENT)
+    if (errno == ENODEV || errno == ENOENT || errno == ENXIO)
       return 0;
     fatal_perror ("Can't read %s", dev);
   }
@@ -584,7 +612,7 @@ void *get1 (drive fd, disk_addr secno)
 {
   static char buf[512];
   
-  if (lseek (fd, secno * 512, 0) < 0)
+  if (llseek (fd, (loff_t) secno * 512, 0) < 0)
     fatal_perror ("lseek %s to sector %d", current_dev, secno);
   if (read (fd, buf, 512) != 512)
     fatal_perror ("read %s sector %d", current_dev, secno);
@@ -594,7 +622,7 @@ void *get1 (drive fd, disk_addr secno)
 
 void put1 (drive fd, disk_addr secno, void *buf)
 {
-  if (lseek (fd, secno * 512, 0) < 0)
+  if (llseek (fd, (loff_t) secno * 512, 0) < 0)
     fatal_perror ("lseek %s to sector %d", current_dev, secno);
   if (write (fd, buf, 512) != 512)
     fatal_perror ("write %s sector %d", current_dev, secno);
@@ -611,7 +639,7 @@ void fatal_perror (char *fmt, ...)
   exit (1);
 }
 
-#endif /* __linux__ */
+#endif /* __unix__ */
 
 #if __dos__
 
